@@ -1,7 +1,7 @@
 use strict;
 package HTTP::BrowserDetect;
-BEGIN {
-  $HTTP::BrowserDetect::VERSION = '1.12';
+{
+  $HTTP::BrowserDetect::VERSION = '1.39';
 }
 
 use vars qw(@ISA @EXPORT @EXPORT_OK @ALL_TESTS);
@@ -12,68 +12,166 @@ require Exporter;
 @EXPORT_OK = qw();
 
 # Operating Systems
-push @ALL_TESTS, qw(
-    win16   win3x       win31
-    win95   win98       winnt
-    windows win32       win2k
-    winxp   win2k3      winvista
-    winme   dotnet      mac
-    macosx  mac68k      macppc
-    os2     unix        sun
-    sun4    sun5        suni86
-    irix    irix5       irix6
-    hpux    hpux9       hpux10
-    aix     aix1        aix2
-    aix3    aix4        linux
-    sco     unixware    mpras
-    reliant dec         sinix
-    freebsd bsd         vms
-    x11     amiga       android
-    win7    ps3gameos   pspgameos
+my @OS_TESTS = qw(
+    windows mac   os2 
+    unix    linux vms 
+    bsd     amiga
+);
+
+# More precise Windows
+my @WINDOWS_TESTS = qw(
+    win16 win3x   win31
+    win95 win98   winnt
+    winme win32   win2k
+    winxp win2k3  winvista 
+    win7  wince   winphone
+);
+
+# More precise Mac
+my @MAC_TESTS = qw(
+    macosx mac68k macppc
+    ios
+);
+
+# More precise Unix
+my @UNIX_TESTS = qw(
+    sun     sun4     sun5
+    suni86  irix     irix5
+    irix6   hpux     hpux9
+    hpux10  aix      aix1
+    aix2    aix3     aix4   
+    sco     unixware mpras
+    reliant dec      sinix
+);
+
+# More precise BSDs
+my @BSD_TESTS = qw(
+    freebsd 
+);
+
+# Gaming devices
+my @GAMING_TESTS = qw(
+    ps3gameos pspgameos
 );
 
 # Devices
-push @ALL_TESTS, qw(
-    palm    audrey      iopener
-    wap     blackberry  iphone
-    ipod    ipad        ps3
-    psp
+my %DEVICE_TESTS = (
+    android => 'Android',
+    audrey => 'Audrey',
+    blackberry => 'BlackBerry',
+    dsi => 'Nintendo DSi',
+    iopener => 'iopener',
+    ipad => 'iPad',
+    iphone => 'iPhone',
+    ipod => 'iPod',
+    kindle => 'Amazon Kindle', 
+    n3ds => 'Nintendo 3DS',
+    palm => 'Palm',
+    ps3  => 'Sony PlayStation 3',
+    psp  => 'Sony PlayStation Portable',
+    wap => 'WAP capable phone',
+    webos => 'webOS',
 );
 
 # Browsers
-push @ALL_TESTS, qw(
-    mosaic      netscape    nav2
-    nav3        nav4        nav4up
-    nav45       nav45up     nav6
-    nav6up      navgold     firefox
-    chrome      safari      ie
+my @BROWSER_TESTS = qw(
+    mosaic        netscape    firefox
+    chrome        safari      ie
+    opera         lynx        links
+    elinks        neoplanet   neoplanet2
+    avantgo       emacs       mozilla     
+    konqueror     r1          netfront
+    mobile_safari
+);
+
+my @IE_TESTS = qw(
     ie3         ie4         ie4up
     ie5         ie5up       ie55
     ie55up      ie6         ie7
-    ie8         opera       opera3
-    opera4      opera5      opera6
-    opera7      lynx        links
+    ie8         ie9         ie10
+);
+
+my @OPERA_TESTS = qw(
+    opera3      opera4     opera5
+    opera6      opera7
+);
+
+my @AOL_TESTS = qw(
     aol         aol3        aol4
-    aol5        aol6        neoplanet
-    neoplanet2  avantgo     emacs
-    mozilla     gecko       r1
-    iceweasel   netfront
+    aol5        aol6        
 );
 
-# Robots
-push @ALL_TESTS, qw(
-    puf         curl        wget
-    getright    robot       yahoo
-    altavista   lycos       infoseek
-    lwp         webcrawler  linkexchange
-    slurp       webtv       staroffice
-    lotusnotes  konqueror   icab
-    google      java        googlemobile
-    msn         msnmobile
+my @NETSCAPE_TESTS = qw(
+    nav2   nav3   nav4  
+    nav4up nav45  nav45up
+    nav6   nav6up navgold
+);
+
+# Firefox variants
+my @FIREFOX_TESTS = qw(
+    firebird    iceweasel   phoenix 
+    namoroka
+);
+
+my @ENGINE_TESTS = qw(
+    gecko    trident
+);
+
+my @ROBOT_TESTS = qw(
+    puf          curl        wget
+    getright     robot       yahoo
+    altavista    lycos       infoseek
+    lwp          webcrawler  linkexchange
+    slurp        webtv       staroffice
+    lotusnotes   icab        google      
+    googlemobile msn         msnmobile
+    facebook
+);
+
+my @MISC_TESTS = qw(
+    mobile      dotnet      x11
+    java
+);
+
+push @ALL_TESTS, (
+    @OS_TESTS,          @WINDOWS_TESTS, @MAC_TESTS,
+    @UNIX_TESTS,        @BSD_TESTS,     @GAMING_TESTS,
+    keys %DEVICE_TESTS, @BROWSER_TESTS, @IE_TESTS, 
+    @OPERA_TESTS,       @AOL_TESTS,     @NETSCAPE_TESTS, 
+    @FIREFOX_TESTS,     @ENGINE_TESTS,  @ROBOT_TESTS,
+    @MISC_TESTS,
+);
+
+
+# Safari build -> version map for versions prior to 3.0
+# (since then, version appears in the user-agent string)
+
+my %safari_build_to_version = qw(
+    48      0.8
+    51      0.8.1
+    60      0.8.2
+    73      0.9
+    74      1.0b2v74
+    85      1.0
+    85.7    1.0.2
+    85.8    1.0.3
+    100     1.1
+    100.1   1.1.1
+    125     1.2
+    125.1   1.2.1
+    125.7   1.2.2
+    125.9   1.2.3
+    125.11  1.2.4
+    312     1.3
+    312.3   1.3.1
+    312.5   1.3.2
+    412     2.0
+    412.5   2.0.1
+    416.12  2.0.2
+    417.8   2.0.3
+    419.3   2.0.4
 );
 
-# Properties
-push @ALL_TESTS, 'mobile';
 
 #######################################################################################################
 # BROWSER OBJECT
@@ -131,7 +229,7 @@ sub _test {
     $self->{tests} = {};
     my $tests = $self->{tests};
 
-    my @ff = qw( firefox firebird iceweasel phoenix );
+    my @ff = ('firefox', @FIREFOX_TESTS );
     my $ff = join "|", @ff;
 
     my $ua = lc $self->{user_agent};
@@ -148,7 +246,7 @@ sub _test {
             ( [^\s]* )              # Beta version string is up to next space
         }x
     );
-    
+
 
     # Firefox version
     if ($ua =~ m{
@@ -157,7 +255,7 @@ sub _test {
                 ( [^.]* )           # Major version number is everything before first dot
                 \.                  # The first dot
                 ( [\d]* )           # Minor version nnumber is digits after first dot
-            }x
+            }xo
         )
     {
         $major = $2;
@@ -189,19 +287,37 @@ sub _test {
 
     }
 
+    # Opera needs to be dealt with specifically
+    # http://dev.opera.com/articles/view/opera-ua-string-changes/
+    # Opera/9.80 (S60; SymbOS; Opera Mobi/320; U; sv) Presto/2.4.15 Version/10.00
+
+    if ( $ua =~ m{\AOpera.*\sVersion/(\d*)\.(\d*)\z}i) {
+        $major = $1;
+        $minor = $2;
+    }
+    elsif ( $ua =~ m{NetFront/(\d*)\.(\d*) Kindle}i ) {
+        $major = $1;
+        $minor = $2;
+    }
+    elsif ( $ua =~ m{Nintendo 3DS;.*\sVersion/(\d*)\.(\d*)}i ) {
+        $major = $1;
+        $minor = $2;
+    }
+
     $major = 0 if !$major;
     $minor = $self->_format_minor( $minor );
 
     # Mozilla browsers
 
     $tests->{GECKO} = ( index( $ua, "gecko" ) != -1 )
-        && ( index( $ua, "khtml, like gecko" ) == -1 );
+        && ( index( $ua, "like gecko" ) == -1 );
 
     $tests->{CHROME} = ( index( $ua, "chrome/" ) != -1 );
     $tests->{SAFARI}
         = (    ( index( $ua, "safari" ) != -1 )
             || ( index( $ua, "applewebkit" ) != -1 ) )
         && ( index( $ua, "chrome" ) == -1 );
+    $tests->{MOBILE_SAFARI} = ($tests->{SAFARI} && index($ua, " mobile safari/") >= 0);
 
     # Chrome Version
     if ( $tests->{CHROME} ) {
@@ -214,45 +330,42 @@ sub _test {
                 ( [^.]* )           # Minor version number is digits after first dot
             }x
         );
-
-        #print "major=$major minor=$minor beta=$beta\n";
     }
 
     # Safari Version
     elsif ( $tests->{SAFARI} ) {
-
-
-        my ( $safari_build, $safari_minor );
-        ( $safari_build, $safari_minor ) = (
+        my ( $safari_build, $safari_minor ) = (
             $ua =~ m{
-                safari
-                \/
-                ( [^.]* )       # Major version number is everything before first dot
-                (?:             # The first dot
-                ( \d* ))?       # Minor version number is digits after first dot
+                safari/
+                ( \d+ )       # Major version number
+                ( \. \d+ )?   # Minor version number is dot and following digits
             }x
         );
 
-# in some obscure cases, extra characters are captured by the regex
-# like: Mozilla/5.0 (SymbianOS/9.1; U; en-us) AppleWebKit/413 (KHTML, like Gecko) Safari/413 UP.Link/6.3.1.15.0
-        $safari_build =~ s{ [^\d] }{}gxms;
-
-        $major = int( $safari_build / 100 );
-        $minor = int( $safari_build % 100 ) / 100;
-        $beta  = $safari_minor;
+        if ( !$safari_build && $ua =~ m{applewebkit\/([\d\.]{1,})}xi ) {
+            # ignore digits after 2nd dot
+            ( $safari_build, $safari_minor ) = split /\./, $1;
+        }
 
+        if ( $safari_build ) {
+            $major = int( $safari_build / 100 );
+            $minor = int( $safari_build % 100 ) / 100;
+            $beta  = $safari_minor;
+        }
     }
 
     # Gecko-powered Netscape (i.e. Mozilla) versions
     $tests->{NETSCAPE}
         = (    !$tests->{FIREFOX}
             && !$tests->{SAFARI}
+            && !$tests->{CHROME}
             && index( $ua, "mozilla" ) != -1
             && index( $ua, "spoofer" ) == -1
             && index( $ua, "compatible" ) == -1
             && index( $ua, "opera" ) == -1
             && index( $ua, "webtv" ) == -1
-            && index( $ua, "hotjava" ) == -1 
+            && index( $ua, "hotjava" ) == -1
+            && index( $ua, "nintendo" ) == -1
             && index( $ua, "playstation 3" ) == -1
             && index( $ua, "playstation portable" ) == -1 );
 
@@ -270,8 +383,6 @@ sub _test {
             }x
         );
         $minor = 0 + ".$minor";
-
-        #print "major=$major minor=$minor beta=$beta\n";
     }
 
     # Netscape browsers
@@ -304,6 +415,8 @@ sub _test {
     $tests->{IE6} = ( $tests->{IE} && $major == 6 );
     $tests->{IE7} = ( $tests->{IE} && $major == 7 );
     $tests->{IE8} = ( $tests->{IE} && $major == 8 );
+    $tests->{IE9} = ( $tests->{IE} && $major == 9 );
+    $tests->{IE10} = ( $tests->{IE} && $major == 10 );
 
     # Neoplanet browsers
 
@@ -329,7 +442,7 @@ sub _test {
     $tests->{OPERA3} = ( index( $ua, "opera 3" ) != -1 )
         || ( index( $ua, "opera/3" ) != -1 );
     $tests->{OPERA4} = ( index( $ua, "opera 4" ) != -1 )
-        || ( index( $ua, "opera/4" ) != -1 );
+        || ( index( $ua, "opera/4" ) != -1 && ( index( $ua, "nintendo dsi" ) == -1 ) );
     $tests->{OPERA5} = ( index( $ua, "opera 5" ) != -1 )
         || ( index( $ua, "opera/5" ) != -1 );
     $tests->{OPERA6} = ( index( $ua, "opera 6" ) != -1 )
@@ -346,20 +459,21 @@ sub _test {
     $tests->{KONQUEROR}  = ( index( $ua, "konqueror" ) != -1 );
     $tests->{LYNX}       = ( index( $ua, "lynx" ) != -1 );
     $tests->{LINKS}      = ( index( $ua, "links" ) != -1 );
+    $tests->{ELINKS}     = ( index( $ua, "elinks" ) != -1 );
     $tests->{WEBTV}      = ( index( $ua, "webtv" ) != -1 );
     $tests->{MOSAIC}     = ( index( $ua, "mosaic" ) != -1 );
-    $tests->{PUF}        = ( index( $ua, "puf" ) != -1 );
+    $tests->{PUF}        = ( index( $ua, "puf/" ) != -1 );
     $tests->{WGET}       = ( index( $ua, "wget" ) != -1 );
     $tests->{GETRIGHT}   = ( index( $ua, "getright" ) != -1 );
     $tests->{LWP}
         = ( index( $ua, "libwww-perl" ) != -1 || index( $ua, "lwp-" ) != -1 );
-    $tests->{YAHOO}  = ( index( $ua, "yahoo" ) != -1 );
+    $tests->{YAHOO}  = ( index( $ua, "yahoo" ) != -1 ) && ( index( $ua, 'jp.co.yahoo.android') == -1 );
     $tests->{GOOGLE} = ( index( $ua, "googlebot" ) != -1 );
     $tests->{GOOGLEMOBILE} = ( index( $ua, "googlebot-mobile" ) != -1 );
     $tests->{MSN} = ( (index( $ua, "msnbot" ) != -1 || index( $ua, "bingbot" )) != -1 );
     $tests->{MSNMOBILE} = ( (index( $ua, "msnbot-mobile" ) != -1 || index( $ua, "bingbot-mobile" )) != -1 );
     $tests->{JAVA}
-        = ( index( $ua, "java" ) != -1 || index( $ua, "jdk" ) != -1 );
+        = ( index( $ua, "java" ) != -1 || index( $ua, "jdk" ) != -1 || index($ua, "jakarta commons-httpclient") != -1);
     $tests->{ALTAVISTA}    = ( index( $ua, "altavista" ) != -1 );
     $tests->{SCOOTER}      = ( index( $ua, "scooter" ) != -1 );
     $tests->{LYCOS}        = ( index( $ua, "lycos" ) != -1 );
@@ -367,6 +481,7 @@ sub _test {
     $tests->{WEBCRAWLER}   = ( index( $ua, "webcrawler" ) != -1 );
     $tests->{LINKEXCHANGE} = ( index( $ua, "lecodechecker" ) != -1 );
     $tests->{SLURP}        = ( index( $ua, "slurp" ) != -1 );
+    $tests->{FACEBOOK}     = ( index( $ua, "facebookexternalhit" ) != -1 );
     $tests->{ROBOT}        = (
         (          $tests->{WGET}
                 || $tests->{PUF}
@@ -383,13 +498,15 @@ sub _test {
                 || $tests->{GOOGLEMOBILE}
                 || $tests->{MSN}
                 || $tests->{MSNMOBILE}
+                || $tests->{FACEBOOK}
+                || $tests->{JAVA}
         )
             || index( $ua, "bot" ) != -1
             || index( $ua, "spider" ) != -1
             || index( $ua, "crawl" ) != -1
             || index( $ua, "agent" ) != -1
-            || index( $ua, "seek" ) != -1
-            || index( $ua, "search" ) != -1
+            || $ua =~ /seek (?! mo (?: toolbar )? \s+ \d+\.\d+ )/x
+            || $ua =~ /search (?! [\w\s]* toolbar \b | bar \b )/x
             || index( $ua, "reap" ) != -1
             || index( $ua, "worm" ) != -1
             || index( $ua, "find" ) != -1
@@ -402,14 +519,17 @@ sub _test {
     $tests->{NETFRONT} = (
            index( $ua, "playstation 3" ) != -1
         || index( $ua, "playstation portable" ) != -1
+        || index( $ua, "netfront" ) != -1
     );
 
     # Devices
 
     $tests->{BLACKBERRY} = ( index( $ua, "blackberry" ) != -1 );
     $tests->{IPHONE}     = ( index( $ua, "iphone" ) != -1 );
+    $tests->{WEBOS}      = ( index( $ua, "webos" ) != -1 );
     $tests->{IPOD}       = ( index( $ua, "ipod" ) != -1 );
     $tests->{IPAD}       = ( index( $ua, "ipad" ) != -1 );
+    $tests->{KINDLE}     = ( index( $ua, "kindle" ) != -1 );
     $tests->{AUDREY}     = ( index( $ua, "audrey" ) != -1 );
     $tests->{IOPENER}    = ( index( $ua, "i-opener" ) != -1 );
     $tests->{AVANTGO}    = ( index( $ua, "avantgo" ) != -1 );
@@ -431,6 +551,8 @@ sub _test {
             || index( $ua, "zetor" ) != -1 );
     $tests->{PS3} = ( index( $ua, "playstation 3" ) != -1 );
     $tests->{PSP} = ( index( $ua, "playstation portable" ) != -1 );
+    $tests->{DSI}   = ( index( $ua, "nintendo dsi" ) != -1 );
+    $tests->{'N3DS'} = ( index( $ua, "nintendo 3ds" ) != -1 );
 
     $tests->{MOBILE} = (
                index( $ua, "up.browser" ) != -1
@@ -469,7 +591,13 @@ sub _test {
             || index( $ua, "android" ) != -1
             || index( $ua, "symbos" ) != -1
             || index( $ua, "opera mobi" ) != -1
+            || index( $ua, "fennec" ) != -1
+            || index( $ua, "opera tablet" ) != -1
             || $tests->{PSP}
+            || $tests->{DSI}
+            || $tests->{'N3DS'}
+            || $tests->{GOOGLEMOBILE}
+            || $tests->{MSNMOBILE}
     );
 
     # Operating System
@@ -504,6 +632,9 @@ sub _test {
     $tests->{WIN7}     = ( index( $ua, "nt 6.1" ) != -1 );
     $tests->{DOTNET}   = ( index( $ua, ".net clr" ) != -1 );
 
+    $tests->{WINCE}    = ( index( $ua, "windows ce" ) != -1 );
+    $tests->{WINPHONE} = ( index( $ua, "windows phone os" ) != -1 );
+
     $tests->{WINME} = ( index( $ua, "win 9x 4.90" ) != -1 );    # whatever
     $tests->{WIN32} = (
         (          $tests->{WIN95}
@@ -531,6 +662,8 @@ sub _test {
                 || $tests->{WINVISTA}
                 || $tests->{WIN7}
                 || $tests->{WINME}
+                || $tests->{WINCE}
+                || $tests->{WINPHONE}
         )
             || index( $ua, "win" ) != -1
     );
@@ -547,6 +680,7 @@ sub _test {
         = (    ( $tests->{MAC} )
             && ( index( $ua, "ppc" ) != -1 || index( $ua, "powerpc" ) != -1 )
         );
+    $tests->{IOS} = $tests->{IPAD} || $tests->{IPOD} || $tests->{IPHONE};
 
     # Others
 
@@ -630,9 +764,19 @@ sub _test {
         }
     }
 
+    # Engines
+
+    $tests->{TRIDENT} = ( index( $ua, "trident/" ) != -1 );
+
+    $self->{engine_version} = $self->{gecko_version};
+
+    if ( $ua =~ /trident\/([\w\.\d]*)/ ) {
+        $self->{engine_version} = $1;
+    }
+
     # RealPlayer
     $tests->{REALPLAYER}
-        = ( index( $ua, "r1" ) != -1 || index( $ua, "realplayer" ) != -1 );
+        = ( index( $ua, "(r1 " ) != -1 || index( $ua, "realplayer" ) != -1 );
 
     $self->{realplayer_version} = undef;
     if ( $tests->{REALPLAYER} ) {
@@ -644,6 +788,15 @@ sub _test {
         }
     }
 
+    # Device from UA
+
+    $self->{device_name} = undef;
+
+    if ( $ua =~ /windows phone os [^\)]+ iemobile\/[^;]+; ([^;]+; [^;\)]+)/g ) {
+        $self->{device_name} = substr $self->{user_agent}, pos($ua) - length $1, length $1;
+        $self->{device_name} =~ s/; / /;
+    }
+
     $self->{major} = $major;
     $self->{minor} = $minor;
     $self->{beta}  = $beta;
@@ -664,11 +817,17 @@ sub browser_string {
         $browser_string = 'Opera'       if $self->opera;
         $browser_string = 'Mosaic'      if $self->mosaic;
         $browser_string = 'Lynx'        if $self->lynx;
+        $browser_string = 'Links'       if $self->links;
         $browser_string = 'RealPlayer'  if $self->realplayer;
         $browser_string = 'IceWeasel'   if $self->iceweasel;
         $browser_string = 'curl'        if $self->curl;
         $browser_string = 'puf'         if $self->puf;
         $browser_string = 'NetFront'    if $self->netfront;
+        $browser_string = 'Mobile Safari' if $self->mobile_safari;
+        $browser_string = 'ELinks'      if $self->elinks;
+        $browser_string = 'BlackBerry'  if $self->blackberry;
+        $browser_string = 'Nintendo 3DS' if $self->n3ds;
+        $browser_string = 'Nintendo DSi' if $self->dsi;
     }
     return $browser_string;
 }
@@ -686,6 +845,7 @@ sub os_string {
         $os_string = 'Win2k3'   if $self->win2k3;
         $os_string = 'WinVista' if $self->winvista;
         $os_string = 'Win7'     if $self->win7;
+        $os_string = 'Windows Phone' if $self->winphone;
         $os_string = 'Mac'      if $self->mac;
         $os_string = 'Mac OS X' if $self->macosx;
         $os_string = 'Win3x'    if $self->win3x;
@@ -694,6 +854,7 @@ sub os_string {
         $os_string = 'Linux'    if $self->linux;
         $os_string = 'Playstation 3 GameOS' if $self->ps3gameos;
         $os_string = 'Playstation Portable GameOS' if $self->pspgameos;
+        $os_string = 'iOS' if $self->iphone || $self->ipod || $self->ipad;
     }
     return $os_string;
 }
@@ -789,31 +950,51 @@ sub public_minor {
 }
 
 sub public_beta {
-
     my ( $self, $check ) = _self_or_default( @_ );
-    return $self->beta( $check );
+    my ( $major, $minor, $beta ) = $self->_public;
 
+    return $beta;
 }
 
 sub _public {
     my ( $self, $check ) = _self_or_default( @_ );
 
-    my $ua = $self->user_agent;
+    # Return Public version of Safari. See RT #48727.
+    if ($self->safari) {
+        my $ua = lc $self->user_agent;
 
-    # this is the Public version of Safari.  See RT #48727
-    if ( $self->safari && $ua =~ m{
+        # Safari starting with version 3.0 provides its own public version
+        if ( $ua =~ m{
                 version/
-                ( [^.]* )       # Major version number is everything before first dot
-                \.              # The first dot
-                ( [^.]* )       # Minor version number is digits after first dot
+                ( \d+ )       # Major version number is everything before first dot
+                ( \. \d+ )?   # Minor version number is first dot and following digits
             }x
-        )
-    {
-        return ( $1, $2 );
+            )
+        {
+            return ($1, $2, undef);
+        }
+
+        # Safari before version 3.0 had only build numbers;
+        # use a lookup table provided by Apple to convert to version numbers
+        if ($ua =~ m{ safari/ ( \d+ (?: \.\d+ )* ) }x) {
+            my $build = $1;
+            my $version = $safari_build_to_version{$build};
+            unless ($version) {
+                # if exact build -> version mapping doesn't exist, find next lower build
+                for my $maybe_build (sort { $b <=> $a } keys %safari_build_to_version) {
+                    $version = $safari_build_to_version{$maybe_build}, last
+                        if $build >= $maybe_build;
+                }
+            }
+            my ($major, $minor) = split /\./, $version;
+            my $beta;
+            $minor =~ s/(\D.*)// and $beta = $1;
+            $minor = 0 + ('.' . $minor);
+            return ( $major, $minor, ($beta ? 1 : undef) );
+        }
     }
 
-    return ( $self->major, $self>minor );
-
+    return ( $self->major, $self->minor, $self->beta($check) );
 }
 
 
@@ -822,12 +1003,16 @@ sub engine_string {
 
     my ( $self, $check ) = _self_or_default( @_ );
 
+    if ( $self->user_agent =~ m{\bKHTML\b} ) {
+        return 'KHTML';
+    }
+
     if ( $self->gecko ) {
         return 'Gecko';
     }
 
-    if ( $self->user_agent =~ m{KHTML, like Gecko} ) {
-        return 'KHTML';
+    if ( $self->trident ) {
+        return 'Trident';
     }
 
     if ( $self->ie ) {
@@ -838,18 +1023,14 @@ sub engine_string {
         return 'NetFront';
     }
 
-    return;
+    return undef;
 }
 
 sub _engine {
 
     my ( $self, $check ) = _self_or_default( @_ );
 
-    if ( $self->gecko ) {
-        return $self->gecko_version;
-    }
-
-    return;
+    return $self->{engine_version};
 
 }
 
@@ -861,7 +1042,7 @@ sub engine_version {
         return $self->engine_major + $self->engine_minor;
     }
 
-    return;
+    return undef;
 
 }
 
@@ -874,7 +1055,7 @@ sub engine_major {
         return shift @version;
     }
 
-    return;
+    return undef;
 
 }
 
@@ -888,7 +1069,7 @@ sub engine_minor {
         return $self->_format_minor( shift @version );
     }
 
-    return;
+    return undef;
 
 }
 
@@ -923,34 +1104,23 @@ sub device {
 
     my ( $self, $check ) = _self_or_default( @_ );
 
-    my @devices = qw(
-        blackberry  iphone  ipod    ipad  ps3  psp
-    );
-
-    foreach my $device ( @devices ) {
+    foreach my $device ( keys %DEVICE_TESTS ) {
         return $device if ( $self->$device );
     }
 
-    return;
+    return undef;
 }
 
 sub device_name {
 
     my ( $self, $check ) = _self_or_default( @_ );
 
-    my %device_name = (
-        blackberry => 'BlackBerry',
-        iphone => 'iPhone',
-        ipod => 'iPod',
-        ipad => 'iPad',
-        psp  => 'Sony PlayStation Portable',
-        ps3  => 'Sony PlayStation 3',
-    );
+    return $self->{device_name} if defined $self->{device_name};
 
     my $device = $self->device;
-    return if !$device;
+    return undef if !$device;
 
-    return $device_name{ $self->device };
+    return $DEVICE_TESTS{ $self->device };
 }
 
 
@@ -960,16 +1130,20 @@ sub _language_country {
 
     if ( $self->safari ) {
         if (   $self->major == 1
-            && $self->user_agent =~ m/\s ( [a-z]{2,} ) \)/xms )
+            && $self->user_agent =~ m/\s ( [a-z]{2} ) \)/xms )
         {
             return { language => uc $1 };
         }
-        if ( $self->user_agent =~ m/([a-z]{2,})-([a-z]{2,})/xms ) {
+        if ( $self->user_agent =~ m/([a-z]{2})-([a-z]{2})/xms ) {
             return { language => uc $1, country => uc $2 };
         }
     }
 
-    if ( $self->user_agent =~ m/\b([a-z]{2,})-([A-Za-z]{2,})\b/xms ) {
+    if ( $self->aol && $self->user_agent =~ m/;([A-Z]{2})_([A-Z]{2})\)/ ) {
+        return { language => $1, country => $2 };
+    }
+
+    if ( $self->user_agent =~ m/\b([a-z]{2})-([A-Za-z]{2})\b/xms ) {
         return { language => uc $1, country => uc $2 };
     }
 
@@ -980,7 +1154,7 @@ sub _language_country {
     if ( $self->user_agent =~ m/\(([^)]+)\)/xms ) {
         my @parts = split(/;/,$1);
         foreach my $part (@parts) {
-            if ($part =~ /^\s*([a-z]{2,})\s*$/) {
+            if ($part =~ /^\s*([a-z]{2})\s*$/) {
                 return { language => uc $1 };
             }
         }
@@ -998,8 +1172,25 @@ sub _format_minor {
 
 }
 
+sub browser_properties {
+
+    my ( $self, $check ) = _self_or_default( @_ );
+
+    my @browser_properties;
+    foreach my $property (keys %{$self->{tests}}) {
+        push @browser_properties, lc($property) if (${$self->{tests}}{$property});
+    }
+
+    # devices are a property too but it's not stored in %tests 
+    # so I explicitly test for it and add it
+    push @browser_properties, 'device' if ($self->device());
+
+    return @browser_properties;
+}
 1;
 
+# ABSTRACT: Determine Web browser, version, and platform from an HTTP user agent string
+
 
 
 =pod
@@ -1010,7 +1201,7 @@ HTTP::BrowserDetect - Determine Web browser, version, and platform from an HTTP
 
 =head1 VERSION
 
-version 1.12
+version 1.39
 
 =head1 SYNOPSIS
 
@@ -1046,19 +1237,7 @@ The HTTP::BrowserDetect object does a number of tests on an HTTP user agent
 string. The results of these tests are available via methods of the object.
 
 This module is based upon the JavaScript browser detection code available at
-B<http://www.mozilla.org/docs/web-developer/sniffer/browser_type.html>.
-
-=head1 INSTALLATION
-
-In most cases, you can just issue the following commands:
-
-  perl Build.PL
-  ./Build
-  ./Build test
-  ./Build install
-
-Please see the documentation for L<Module::Build> if you have questions about
-installing to custom locations etc.
+L<http://www.mozilla.org/docs/web-developer/sniffer/browser_type.html>.
 
 =head1 CONSTRUCTOR AND STARTUP
 
@@ -1096,15 +1275,26 @@ be in the form of an upper case 2 character code. ie: EN, DE, etc
 =head2 device()
 
 Returns the method name of the actual hardware, if it can be detected.
-Currently returns one of: blackberry, iphone, ipod or ipad  Returns UNDEF if
+Currently returns one of: android, audrey, blackberry, dsi, iopener, ipad, 
+iphone, ipod, kindle, n3ds, palm, ps3, psp, wap, webos. Returns C<undef> if 
 no hardware can be detected
 
 =head2 device_name()
 
 Returns a human formatted version of the hardware device name.  These names
 are subject to change and are really meant for display purposes.  You should
-use the device() method in your logic.  Returns one of: BlackBerry, iPhone,
-iPod or iPad.  Returns UNDEF if no hardware can be detected.
+use the device() method in your logic.  Returns one of: Android, Audrey, 
+BlackBerry, Nintendo DSi, iopener, iPad, iPhone, iPod, Amazon Kindle, Nintendo
+3DS, Palm, Sony PlayStation 3, Sony Playstation Portable, WAP capable phone, 
+webOS. Also Windows-based smartphones will output various different names like
+HTC T7575. Returns C<undef> if this is not a device or if no device name can 
+be detected.
+
+=head2 browser_properties()
+
+Returns a list of the browser properties, that is, all of the tests that passed
+for the provided user_agent string. Operating systems, devices, browser names,
+mobile and robots are all browser properties.
 
 =head1 Detecting Browser Version
 
@@ -1176,24 +1366,27 @@ is thrown away.
 
 Returns one of the following:
 
-Gecko, KHTML, MSIE, NetFront
+Gecko, KHTML, Trident, MSIE, NetFront
 
-Returns undef if no string can be found.
+Returns C<undef> if no string can be found.
 
 =head2 engine_version()
 
 Returns the version number of the rendering engine. Currently this only
-returns a version number for Gecko. Returns undef for all other engines.
+returns a version number for Gecko and Trident. Returns C<undef> for all
+other engines.
 
 =head2 engine_major()
 
 Returns the major version number of the rendering engine. Currently this only
-returns a version number for Gecko. Returns undef for all other engines.
+returns a version number for Gecko and Trident. Returns C<undef> for all
+other engines.
 
 =head2 engine_minor()
 
 Returns the minor version number of the rendering engine. Currently this only
-returns a version number for Gecko. Returns undef for all other engines.
+returns a version number for Gecko and Trident. Returns C<undef> for all
+other engines.
 
 =head1 Detecting OS Platform and Version
 
@@ -1209,12 +1402,14 @@ winnt, which is a type of win32)
         winme win95 win98
         winnt
             win2k winxp win2k3 winvista win7
+    wince
+    winphone
 
 =head2 dotnet()
 
 =head2 mac()
 
-mac68k macppc macosx
+mac68k macppc macosx ios
 
 =head2 os2()
 
@@ -1241,8 +1436,9 @@ distinguish between Win95 and WinNT.
 Returns one of the following strings, or undef. This method exists solely for
 compatibility with the L<HTTP::Headers::UserAgent> module.
 
-  Win95, Win98, WinNT, Win2K, WinXP, Win2K3, WinVista, Win7, Mac, Mac OS X,
-  Win3x, OS2, Unix, Linux, Playstation 3 GameOS, Playstation Portable GameOS
+  Win95, Win98, WinNT, Win2K, WinXP, Win2k3, WinVista, Win7, Windows Phone,
+  Mac, Mac OS X, iOS, Win3x, OS2, Unix, Linux, Playstation 3 GameOS,
+  Playstation Portable GameOS
 
 =head1 Detecting Browser Vendor
 
@@ -1250,67 +1446,68 @@ The following methods are available, each returning a true or false value.
 Some methods also test for the browser version, saving you from checking the
 version separately.
 
-=head3 netscape nav2 nav3 nav4 nav4up nav45 nav45up navgold nav6 nav6up
-
-=head3 gecko
-
-=head3 mozilla
-
-=head3 firefox
-
-=head3 safari
+=head3 aol aol3 aol4 aol5 aol6
 
 =head3 chrome
 
-=head3 ie ie3 ie4 ie4up ie5 ie55 ie6 ie7 ie8
-
-=head3 neoplanet neoplanet2
-
-=head3 mosaic
-
-=head3 aol aol3 aol4 aol5 aol6
-
-=head3 webtv
-
-=head3 opera opera3 opera4 opera5 opera6 opera7
-
-=head3 lynx links
+=head3 curl
 
 =head3 emacs
 
-=head3 staroffice
+=head3 firefox
 
-=head3 lotusnotes
+=head3 gecko
 
 =head3 icab
 
-=head3 konqueror
+=head3 ie ie3 ie4 ie4up ie5 ie55 ie6 ie7 ie8 ie9 ie10
 
 =head3 java
 
-=head3 curl
+=head3 konqueror
 
-=head3 realplayer
+=head3 lotusnotes
+
+=head3 lynx links elinks
+
+=head3 mobile_safari
+
+=head3 mosaic
+
+=head3 mozilla
+
+=head3 neoplanet neoplanet2
 
 =head3 netfront
 
-Netscape 6, even though its called six, in the userAgent string has version
-number 5. The nav6 and nav6up methods correctly handle this quirk. The firefox
-text correctly detects the older-named versions of the browser (Phoenix,
-Firebird)
+=head3 netscape nav2 nav3 nav4 nav4up nav45 nav45up navgold nav6 nav6up
+
+=head3 opera opera3 opera4 opera5 opera6 opera7
+
+=head3 realplayer
+
+=head3 safari
+
+=head3 staroffice
+
+=head3 webtv
+
+Netscape 6, even though its called six, in the User-Agent string has version
+number 5. The nav6 and nav6up methods correctly handle this quirk. The Firefox
+test correctly detects the older-named versions of the browser (Phoenix,
+Firebird).
 
 =head2 browser_string()
 
 Returns undef on failure.  Otherwise returns one of the following:
 
-Firefox, Safari, Chrome, MSIE, etc
-
-To see a complete list of possible browser strings, check the browser_string()
-method in the source code.
+Netscape, Firefox, Safari, Chrome, MSIE, WebTV, AOL Browser, Opera, Mosaic,
+Lynx, Links, ELinks, RealPlayer, IceWeasel, curl, puf, NetFront, Mobile Safari, 
+BlackBerry
 
 =head2 gecko_version()
 
-If a Gecko rendering engine is used (as in Mozilla or Firebird), returns the
+If a Gecko rendering engine is used (as in Mozilla or Firefox), returns the
 version of the renderer (e.g. 1.3a, 1.7, 1.8) This might be more useful than
 the particular browser name or version when correcting for quirks in different
 versions of this rendering engine. If no Gecko browser is being used, or the
@@ -1328,6 +1525,8 @@ The following methods are available, each returning a true or false value.
 
 =head3 blackberry
 
+=head3 dsi
+
 =head3 iopener
 
 =head3 iphone
@@ -1336,8 +1535,14 @@ The following methods are available, each returning a true or false value.
 
 =head3 ipad
 
+=head3 kindle
+
+=head3 n3ds
+
 =head3 palm
 
+=head3 webos
+
 =head3 wap
 
 =head3 psp
@@ -1356,34 +1561,36 @@ other automated Web client.
 The following additional methods are available, each returning a true or false
 value. This is by no means a complete list of robots that exist on the Web.
 
-=head3 wget
-
-=head3 getright
-
-=head3 yahoo
-
 =head3 altavista
 
-=head3 lycos
+=head3 facebook
 
-=head3 infoseek
-
-=head3 lwp
-
-=head3 webcrawler
-
-=head3 linkexchange
-
-=head3 slurp
+=head3 getright
 
 =head3 google
 
 =head3 googlemobile
 
+=head3 infoseek
+
+=head3 linkexchange
+
+=head3 lwp
+
+=head3 lycos
+
 =head3 msn (same as bing)
 
 =head3 puf
 
+=head3 slurp
+
+=head3 webcrawler
+
+=head3 wget
+
+=head3 yahoo
+
 =head1 CREDITS
 
 Lee Semel, lee@semel.net (Original Author)
@@ -1396,6 +1603,8 @@ Olaf Alders, C<olaf at wundercounter.com> (co-maintainer)
 
 Thanks to the following for their contributions:
 
+cho45
+
 Leonardo Herrera
 
 Denis F. Latypoff
@@ -1432,26 +1641,55 @@ Jay Rifkin
 
 Luke Saunders
 
+Jacob Rask
+
+Heiko Weber
+
+Jon Jensen
+
+Jesse Thompson
+
+Graham Barr
+
+Enrico Sorcinelli
+
+Olivier Bilodeau
+
+Yoshiki Kurihara
+
+Paul Findlay
+
+Uwe Voelker
+
+Douglas Christopher Wilson
+
+John Oatis
+
+Atsushi Kato
+
+Ronald J. Kimball
+
 =head1 TO DO
 
-The _engine() method currently only handles Gecko.  It needs to be expanded to
-handle other rendering engines.
+The C<_engine()> method currently only handles Gecko and Trident.  It
+needs to be expanded to handle other rendering engines.
 
 POD coverage is also not 100%.
 
-If you're able to help out with anything on the TO DO list, please do. A great
-deal of the latest improvements have come from patches via RT and GitHub pull
-requests. If you're able to submit changes via GitHub, I'm generally able to
-get them into a new release fairly quickly.
-
 =head1 SEE ALSO
 
-"The Ultimate JavaScript Client Sniffer, Version 3.0", B<http://www.mozilla.org/docs/web-developer/sniffer/browser_type.html>.
+"The Ultimate JavaScript Client Sniffer, Version 3.0", L<http://www.mozilla.org/docs/web-developer/sniffer/browser_type.html>
 
-"Browser ID (User-Agent) Strings" B<http://www.zytrax.com/tech/web/browser_ids.htm>
+"Browser ID (User-Agent) Strings", L<http://www.zytrax.com/tech/web/browser_ids.htm>
+
+Safari "Historical User Agent strings", L<http://developer.apple.com/internet/safari/uamatrix.html> (now gone, retrieved 2007-06-20)
+
+"Safari Agent Strings", L<http://homepage.mac.com/jprince/designSandbox/web/safari-agents/>
 
 perl(1), L<HTTP::Headers>, L<HTTP::Headers::UserAgent>.
 
+=head1
+
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
@@ -1466,9 +1704,9 @@ You can also look for information at:
 
 L<http://github.com/oalders/http-browserdetect>
 
-=item * RT: CPAN's request tracker
+=item * Reporting Issues
 
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=HTTP-BrowserDetect>
+L<https://github.com/oalders/http-browserdetect/issues>
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
@@ -1480,7 +1718,7 @@ L<http://cpanratings.perl.org/d/HTTP-BrowserDetect>
 
 =item * Search CPAN
 
-L<http://search.cpan.org/dist/HTTP-BrowserDetect/>
+L<http://beta.metacpan.org/module/HTTP::BrowserDetect>
 
 =back
 
@@ -1489,18 +1727,42 @@ L<http://search.cpan.org/dist/HTTP-BrowserDetect/>
 The biggest limitation at this point is the test suite, which really needs to
 have many more UserAgent strings to test against.
 
+=head1 CONTRIBUTING
+
 Patches are certainly welcome, with many thanks for the excellent
 contributions which have already been received. The preferred method of
-patching would be to fork the GitHub repo and then send me a pull requests,
+patching would be to fork the GitHub repo and then send me a pull request,
 but plain old patch files are also welcome.
 
-=head1 AUTHOR
+If you're able to add test cases, this will speed up the time to release your
+changes. Just edit t/useragents.json so that the test coverage includes any
+changes you have made. Please contact me if you have any questions.
+
+This distribution uses L<Dist::Zilla>. If you're not familiar with this
+module, please see L<https://github.com/oalders/http-browserdetect/issues/5>
+for some helpful tips to get you started.
+
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Lee Semel <lee@semel.net>
+
+=item *
+
+Peter Walsham
+
+=item *
 
 Olaf Alders <olaf@wundercounter.com> (current maintainer)
 
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Lee Semel.
+This software is copyright (c) 2011 by Lee Semel.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -1510,5 +1772,3 @@ the same terms as the Perl 5 programming language system itself.
 
 __END__
 
-# ABSTRACT: Determine Web browser, version, and platform from an HTTP user agent string
-
